# encoding: UTF-8

$LOAD_PATH << '.'

require 'json'
require 'htmlentities'
require 'file_facilities'
require 'filter_tweet_data'

def get_file_class_name file_name
  class_name = file_name.match(/(.+)\.json/)[1]
  class_name
end

def calc_tweet_statistics file_name
  coder = HTMLEntities.new
  hash = Hash.new(0)
  rts = 0
  https = 0
  username = 0
  characters_length = 0
  File.open(file_name, 'r') do |file| 
    while (line = file.gets)
      # For each line it gathers the Twitter user name and its correspondent tweet text
      tweet = JSON.parse(line)
      tweet_decoded_text = FilterTweetData.filter_tweet(coder.decode(tweet['text']))
      if tweet_decoded_text.match(/RT/)
        rts += 1
      end
      if tweet_decoded_text.match(/http:/)
        https += 1
      end
      if tweet_decoded_text.match(/@[^\s]*/)
        username += 1
      end
      tweet_decoded_text = FilterTweetData.tweet_processing(tweet_decoded_text)
      characters_length += tweet_decoded_text.length
      tweet_decoded_text = FilterTweetData.remove_numbers_quote_characters(tweet_decoded_text)
      # tweet_decoded_text.gsub(/#[[:alpha:][:digit:]]+/) do |pattern|
      tweet_decoded_text.downcase.split.each do |pattern|
        hash[pattern.downcase] += 1
      end
    end
  end
  [hash, rts, https, username, characters_length]
end

def write_tweet_statistics file_name, hash, rts, https, username, characters_length
  f = File.open("#{file_name}_statistics.txt", 'w')
  nr_tweets = IO.readlines("#{file_name}.json").length

  f.puts "#{username} usuários: #{sprintf("%.2f", (username/nr_tweets.to_f)*100)}%"
  f.puts "#{rts} retweets: #{sprintf("%.2f", (rts/nr_tweets.to_f)*100)}%"
  f.puts "#{https} links http na classe: #{sprintf("%.2f", (https/nr_tweets.to_f)*100)}%"
  f.puts "Tamanho médio dos tuítes: #{characters_length/nr_tweets.to_f} "

  sorted_hash = hash.sort_by {|k, v| v}.reverse
  sorted_hash.each {|key, value| f.puts "#{key}: #{value}"}

  f.close
end


coder = HTMLEntities.new

directories = FileFacilities.subdirectories(ARGV[0])

directories.each do |directory|
  json_files = FileFacilities.class_files(directory)
  json_files.each do |json_file|
    hash, rts, https, username, characters_length = calc_tweet_statistics(json_file)
    file_name = json_file.gsub(/.json/, '')
    write_tweet_statistics(file_name, hash, rts, https, username, characters_length)
  end
end
