#!/usr/bin/env ruby
# encoding: UTF-8

# The file names are the class name or contain a subject that belongs to some class.
# All classes are considered mutually exclusive, but there are 2 classes that have intersection of subjects.
# Those are: cybersecurity_threats_general_security_news and general_security_news_cybersecurity_news
# The difference between them is their type of vector agent. In the first class a cybersecurity agent (person,
# entity, etc) give birth to their news while the second one has a general security agent. 

# The classified_tweets_last_run class only store the tweets classified in the last time this script was run.

# To run the script you must be in the 'programacao-dados/scripts' folder and
# type : ruby classify-tweets.rb tweets_security_dataset.json
# tweets_security_dataset.json is the file that contains the tweets to classify
# The json class files are stored in the 'tweets_dataset' folder as you can see in the code below

$LOAD_PATH << '.'

require 'rubygems'
require 'json'
require 'htmlentities'
require 'file_facilities'
require 'filter_tweet_data'


def print_instructions(file_names)
  file_options = ""
  file_names.each_with_index {|f, i| file_options += "#{i+1}  -  #{f.match(/(.*\/)+(.*)\.json/)[2]}\n"}

  instructions = "You are going to classify text that will appear in the output.\n" +
                "For each text you must type the number which corresponds to the class or subject you think it belongs to.\n" +
                "#{file_options}" +
                "If you want to delete a tweet from the dataset without label it type '0'\n" +
                "If you want to create a new class or subject type 'new file', enter and the class or subject file name (e.g., new_subject)\n" +
                "If you don't want to label ths selected tweet type 'enter' to label another random tweet\n" +
                "exit - finish the classifying process and finish the script\n\n"
  print instructions
end

def open_tweet_links(tweet)
  # Open the web pages at the tweet in the google chrome browser
  if (webpages = tweet['text'].scan(/https?:\/\/[a-z0-9A-Z\/\.]*[a-z0-9A-Z]/))
    webpages.each do |webpage|
      `firefox -private-window #{webpage}`
      # This error may happen when google-chrome opens a web page 
      # https://code.google.com/p/chromium/issues/detail?id=351804
      # `firefox -P Incognito -no-remote #{webpage}`
    end
  end
end


def create_subject(files, file_names, tweet_directory)
  subject_name = STDIN.gets.chomp

  files << File.open("#{tweet_directory}/#{subject_name}.json", "a")
  file_names << "#{tweet_directory}/#{subject_name}.json"
end

def label_tweet(tweet, files, file_names, tweet_directory)

  while true
    option = STDIN.gets.chomp
    case option
      when ((1.to_s)..(files.length.to_s))
        files[option.to_i-1].write("#{tweet}")
        break
      when "0"
        return "0"
      when "new file"
        create_subject(files, file_names, tweet_directory)
        files[-1].write("#{tweet}")
        print_instructions(file_names)
        break
      when ""
        return false
      when "exit"
        return "exit"
      else
        puts "You typed a wrong option. Type a valid option."
        print_instructions(file_names)
        redo
    end
  end

  return true
end



files = []
file_names = FileFacilities.class_files(ARGV[1])
file_names.each do |file|
  files << File.open(file, "a")
end


coder = HTMLEntities.new

# Tweets being classified in this execution
last_classified_tweets = File.new("#{ARGV[1]}/classified_tweets_last_run.json", "w")

# File with tweets to classify
tweets = IO.readlines(ARGV[0])
LINES = tweets.size

print_instructions(file_names)

# Find the number of lines and sort a number to classify. Search for it in a hash.
# If the line_number is the hash sort another number, else insert the line_number
# in the hash as a key. After read the tweet and classify it.
classified_tweets = {}

srand()

i = 1
while true

  begin
    line_number = rand(1..LINES)
    if classified_tweets.has_key?(line_number-1)
      redo
    else
      tweet = tweets[line_number-1]
      classified_tweets[line_number-1] = nil
    end
  end

  # UTILIZAR DECODIFICADOR DE HTML
  parsed_tweet = JSON.parse(tweet)
  if FilterTweetData.tweet_processing(coder.decode(parsed_tweet['text'])).length < 35
    next
  end

  puts "Random number   -   #{line_number-1}"
  print "Tweet #{i}\n#{coder.decode(parsed_tweet['text'])}\n"

  open_tweet_links(parsed_tweet)

  case label_tweet(tweet, files, file_names, ARGV[1])
    when true
      i += 1
      last_classified_tweets.write(tweet)
      tweets[line_number-1] = ''   
    when "0"
      tweets[line_number-1] = ''   
    when "exit"
      break
  end

end

files.each {|file| file.close}

File.open(ARGV[0], "w") do |file|
  tweets.each do |line|
    file.print line
  end
end

# Old way to remove the classified tweets from the tweets_dataset (population of data or huge sample) file
# `touch #{FILE_FOLDER}/remanining_tweets_to_classify.json`
# system("diff --unchanged-line-format="" #{ARGV[0]} #{classes[:classified_tweets]} > #{FILE_FOLDER}/remanining_tweets_to_classify.json")
# `rm #{ARGV[0]}`
# `mv #{FILE_FOLDER}/remanining_tweets_to_classify.json #{ARGV[0]}`



# When you want to encode json document to string do JSON.dump(doc)
# The document will look like the original one except for the '/' that won't be scaped by '\'
# just write that in the file and it will be ok: JSON.dump(var.gsub(///, '\/'))